/*
 *
 * (C) Jochen Karrer March 1996
 *
 */

#ifndef MV1DRV
#define MV1DRV


typedef u_char MV1_Event;

typedef struct {
        u_long sx,sy;          /* start */
        u_long dx,dy;  	       /* width, height */
} MV1_Frame;

typedef struct {
        u_long startaddr;        // address in onboard VRAM  
	char interlaced;
	char start_even;
	u_long field_offset;      // offset off second field 
        u_long sx,sy;            // Start coordinate for the grab 
        u_long dx,dy;           // Size of the grabbed Image 
	u_long f_start, s_start; // virtual address off first/second Field 
} MV1_GrabWindow;

typedef struct sdc_s {
	u_long len;
	u_long src;
	u_long dst;
	u_long cksum;
} transf_dscr; // Transfer descriptor

typedef struct MV1_TransferList_s  {
        int len;
        transf_dscr td[2047];
} MV1_TransferList;

#define MV1_TRANSFER_QUEUE_LEN 8

#ifdef __KERNEL__
#define MV1_EVENTQUEUE_SIZE (1<<8)

#define MV1_SEND_EVENT(mv1,X) { \
	static u_long  lastarg=~0L;\
	if(lastarg != arg){ \
		lastarg = arg; \
		mv1_send_event(mv1,X);\
	}\
}

struct MV1000;
typedef struct  {
	struct MV1000 *mv1;
	char lut;
	int	event_rp;
	MV1_Frame frame;
	u_long last_read_frame;
} mv1_file_data;


typedef struct {


} mv1_operations;

typedef struct {
	u_int rchip,wchip; // read write from chip x
	u_int reg;
	u_int offset;	 // offset of register in struct MV1000
	u_int backstore; // offset of backing store field
	u_int andmask;
	u_int ormask;   // new value to write to the register;
	u_int event;	// written to the event-queue = (old) ioctl number
	u_int cmd;	// read/write
	u_int val;	// new value for backingstore / returnval for read
} mv1_iojob; 
#endif

#define MV1_READ	0
#define MV1_WRITE 	1

#define XC3195 		1
#define DAC8800 	2
#define LM1882  	3
#define ICS1522 	4
#define VCTL		5
#define SSI_32F8102	6

#define MV1_MAJOR 26
#define MV1_CONTROL_MINOR    0

#define MV1_RAW_MINOR	48
#define CCD_INT_MODE_CONT       0
#define CCD_INT_MODE_SINGLE     1
#define FIELD_EVEN 	0
#define FIELD_ODD 	1

/* raw device ioctls */
#define MV1_SET_ZOOM		0x34789

/* control device ioctls */
#define MV1_EVENT_SEND		0x73212
#define GET_BUS_ADDRESS		0x32111

/* MV1000-Events */
#define MV1_FRAME_END_WAIT	0x1
#define MV1_CONFIG_NOTIFY_WAIT  0x2
#define MV1_BAD_IRQ		0x84

/*  Register RW-ioctls	*/
#define MV1_SET_H_START                 0x80
#define MV1_SET_H_END                   0x81
#define MV1_SET_V_START                 0x82
#define MV1_SET_V_END                   0x83
#define MV1_SET_EQ_REMOVE               0x85
#define MV1_SET_GRAB_START_ADDR         0x86
#define MV1_SET_GRAB_CONTROL            0x87
#define MV1_SET_MISC_CONT_STATUS        0x88
#define MV1_SET_LUT_ADDR                0x89
#define MV1_SET_LUT_DATA                0x8a
#define MV1_SET_LUT_CTL                 0x8b
#define MV1_SET_VIDEO_CTL               0x8c
#define MV1_INQUIRE_H_START             0x8d
#define MV1_INQUIRE_H_END               0x8e
#define MV1_INQUIRE_V_START             0x8f
#define MV1_INQUIRE_V_END               0x90
#define MV1_INQUIRE_CUR_LINE            0x91
#define MV1_INQUIRE_EQ_REMOVE           0x92 
#define MV1_INQUIRE_GRAB_START_ADDR     0x93
#define MV1_INQUIRE_GRAB_CONTROL        0x94
#define MV1_INQUIRE_MISC_CONT_STATUS    0x95
#define MV1_INQUIRE_LUT_ADDR            0x96
#define MV1_INQUIRE_LUT_DATA            0x97
#define MV1_INQUIRE_LUT_CTL             0x98
#define MV1_INQUIRE_VIDEO_CTL           0x99


/* 	Lib-ioctls 	*/


#define MV1_SET_CURRENT_CAMERA			0x05
#define MV1_SELECT_PIXELMODE			0x07
#define MV1_INQUIRE_PIXELMODE			0x107
#define MV1_INQUIRE_PIXELBITS			0x09
#define MV1_SELECT_TRANSFER_MODE		0x0a
#define MV1_SET_GP_BIT0				0x14
#define MV1_INQUIRE_GP_BIT0			0x114
#define MV1_SET_GP_BIT1				0x26
#define MV1_INQUIRE_GP_BIT1			0x126
#define MV1_SET_CAMERA_CONFIG			0x0c
#define MV1_SET_CVIDEO_SOURCE			0x0f
#define MV1_INQUIRE_CVIDEO_SOURCE		0x10
#define MV1_SET_CSYNC_SOURCE			0x11
#define MV1_INQUIRE_CSYNC_SOURCE		0xff
#define MV1_SELECT_CLOCK_SOURCE			0x12
#define MV1_INQUIRE_CLOCK_SOURCE		0xfe
#define MV1_SELECT_SYNC_STRIP_RANGE		0x13
#define MV1_INQUIRE_SYNC_STRIP_RANGE		0xfc
#define MV1_SET_SHIFT_DOWN			0x0e
#define MV1_INQUIRE_SHIFT_DOWN			0x10e
#define MV1_SET_PLL_PHASE_TRIM			0x15
#define MV1_INQUIRE_PLL_PHASE_TRIM		0x115
#define MV1_PROGRAMM_PLL			0x16
#define MV1_RD_VSYNC_INFO			0x18
#define MV1_SET_AD_TOP_LEVEL			0xef
#define MV1_INQUIRE_AD_TOP_LEVEL		0xee
#define MV1_SET_AD_BOTTOM_LEVEL			0xed
#define MV1_INQUIRE_AD_BOTTOM_LEVEL		0xec
#define MV1_INQUIRE_FIELD			0x1b
#define MV1_IS_FRAME_END			0x1c
#define MV1_SET_CLAMP_LEVEL			0x1d
#define MV1_INQUIRE_CLAMP_LEVEL 		0xfb
#define MV1_SELECT_CLAMPING			0x1e
#define MV1_INQUIRE_CLAMPING			0x11e
#define MV1_SET_DIG_CAMERA_CONFIG		0x1f
#define MV1_SETCOLOR_MODULE_CONFIG		0x20
#define MV1_SET_PLL_PHASE_DETECTOR		0x21
#define MV1_SET_LSEXP_REDUCTION			0x22
#define MV1_SET_GAIN				0x24
#define MV1_GET_GAIN_RANGE			0x25
#define MV1_GET_OFFSET_RANGE			0x127
#define MV1_SET_DC_OFFSET			0x29
#define MV1_INQUIRE_DC_OFFSET			0x129
#define MV1_SET_EXT_TRIGGER			0x2a
#define MV1_INQUIRE_EXT_TRIGGER			0x12a
#define MV1_SET_TRIGGER_POLARITY		0x2e
#define MV1_INQUIRE_TRIGGER_POLARITY		0x12e
#define MV1_SET_INTERLACE			0x9b
#define MV1_INQUIRE_INTERLACE			0x19b
#define MV1_WAIT_EXT_TRIGGER			0x2b
#define MV1_SET_DIG_CAMERA_CONTROL		0x2c
#define MV1_SET_COLOR_MODULE_CONTROL		0x2d
#define MV1_WAIT_GRAB_DONE			0x2f
#define MV1_SET_GRAB_WINDOW			0x30
#define MV1_INQUIRE_GRAB_WINDOW			0x130
#define MV1_SET_FRAME				0x33
#define MV1_INQUIRE_FRAME		        0x133

#define MV1_INQUIRE_GRAB_WINDOW_SIZE		0x31
#define MV1_INQUIRE_MAX_GRAB_WINDOW_SIZE	0x32
#define MV1_FRAME_FILL				0x34
#define MV1_FRAME_LINE_READ			0x35
#define MV1_FRAME_LINE_WRITE			0x36
#define MV1_FRAME_READ				0x37
#define MV1_FRAME_READ_FIELD			0x38
#define MV1_FRAME_WRITE 			0x39
#define MV1_SET_HOST_MASK			0xf0
#define MV1_INQUIRE_HOST_MASK			0xf1
#define MV1_SET_GRAB_MASK			0xf2
#define MV1_INQUIRE_GRAB_MASK			0xf3 
#define MV1_SET_FILTER_CUTOFF			0x3d
#define MV1_INQUIRE_FILTER_CUTOFF		0x13d
#define MV1_SET_HIGH_FREQ_BOOST			0x3e
#define MV1_INQUIRE_HIGH_FREQ_BOOST		0x13e
#define MV1_SET_GENERAL_PURPOSE_BIT		0x3f
#define MV1_RESET_GENERAL_PURPOSE_BIT		0x40
#define MV1_SET_LUT_MODE			0x41
#define MV1_SELECT_LUT_BANK			0x42
#define MV1_INQUIRE_LUT_BANK			0x4f
#define MV1_LUT_READ				0x43
#define MV1_LUT_WRITE				0x44
#define MV1_WRITE_8BIT_LUT			0x10044
#define MV1_LUT_WRITE_VALUE			0x50 

#define MV1_SET_GRAB				0x49
#define MV1_INQUIRE_GRAB			0x149
#define MV1_SET_GRABFLAG			0x45
#define MV1_INQUIRE_GRABFLAG			0x145
#define MV1_SET_STARTFIELD			0x46
#define MV1_INQUIRE_STARTFIELD			0x146

	#define MV1_SINGLE_GRAB 0x0
	#define MV1_CONT_GRAB   0x1
	#define MV1_GRAB_EVEN   0x0
	#define MV1_GRAB_ODD    0x1
	#define MV1_GRAB_NEXT	0x2

#define MV1_READ_FIELD_LINE_COUNTER		0x4b
#define MV1_SET_SCALE_FACTOR			0x4c
#define MV1_SET_VSCALE_FACTOR			0x4d
#define MV1_SET_HSCALE_FACTOR			0x4e
#define MV1_GRAB_FIELD				0x51
#define MV1_START_GRAB_FIELD			0x52
#define ICS_SET_FBK_DIV				0x70
#define ICS_INQUIRE_FBK_DIV			0x170
#define ICS_SET_REF_DIV				0x71
#define ICS_INQUIRE_REF_DIV			0x171
#define ICS_SET_REF_POL				0x72
#define ICS_INQUIRE_REF_POL			0x172
#define ICS_SET_VCO_GAIN			0x73
#define ICS_INQUIRE_VCO_GAIN			0x173 
#define ICS_SET_PFD_GAIN			0x74
#define ICS_INQUIRE_PFD_GAIN			0x174
#define ICS_SET_PDEN				0x75
#define ICS_INQUIRE_PDEN			0x175
#define ICS_SET_CLKSEL				0x76
#define ICS_INQUIRE_CLKSEL			0x176
#define ICS_SET_PDA				0x77
#define ICS_INQUIRE_PDA				0x177
#define ICS_SET_PDB				0x78
#define ICS_INQUIRE_PDB				0x178
#define ICS_SET_LEAD_LAG			0x9a
#define ICS_INQUIRE_LEAD_LAG			0x19a

#define LM1882_SELECT_SYNC_OUTPUT		0x17
#define LM1882_INQUIRE_SYNC_OUTPUT 		0x117
#define LM1882_SELECT_EQSERR     		0x9e
#define LM1882_INQUIRE_EQSERR   		0x19e
#define LM1882_SELECT_VCSYNC_POLARITY		0x9c
#define LM1882_INQUIRE_VCSYNC_POLARITY		0x19c
#define LM1882_SELECT_HSYNC_POLARITY		0x9d
#define LM1882_INQUIRE_HSYNC_POLARITY		0x19d
#define LM1882_SET_PIXEL_PER_LINE		0x9f
#define LM1882_INQUIRE_PIXEL_PER_LINE		0x19f
#define LM1882_SET_LINE_PER_FRAME		0xa0
#define LM1882_INQUIRE_LINE_PER_FRAME		0x1a0

#define MV11_SET_PULSE_MODE			0xa1
#define MV11_INQUIRE_PULSE_MODE			0x1a1
#define MV11_SELECT_CLOCK			0xa2
#define MV11_INQUIRE_CLOCK			0x1a2
#define MV11_SET_COUNTER			0xa3
#define MV11_INQUIRE_COUNTER			0x1a3
#define MV11_SET_TRIG_ENABLE			0xa4
#define MV11_INQUIRE_TRIG_ENABLE		0x1a4
#define MV11_SET_TRIG_POL			0xa5
#define MV11_INQUIRE_TRIG_POL			0x1a5
#define MV11_SET_LS_EXP				0xa6
#define MV11_INQUIRE_LS_EXP			0x1a6
#define MV11_SET_LDV_POL			0xa7
#define MV11_INQUIRE_LDV_POL			0x1a7
#define MV11_SET_FDV_POL			0xa8
#define MV11_INQUIRE_FDV_POL			0x1a8
#define MV11_SET_PCLK_POL			0xa9
#define MV11_INQUIRE_PCLK_POL			0x1a9
#define MV11_SET_BYPASS_COUNTER			0xaa
#define MV11_INQUIRE_BYPASS_COUNTER		0x1aa
#define MV11_SELECT_CAMERA_DRIVE		0xab
#define MV11_INQUIRE_CAMERA_DRIVE		0x1ab
#define MV11_SET_MODE_CONTROL			0xac
#define MV11_INQUIRE_MODE_CONTROL		0x1ac
#define MV11_SELECT_PAR_FRAME_REMOVE		0xad
#define MV11_INQUIRE_PAR_FRAME_REMOVE		0x1ad
#define MV1_SET_TOP_SC				0xae
#define MV1_INQUIRE_TOP_SC			0x1ae


#define MV1_SET_INTEGRATION_TIME		0x53
#define CAM_SET_INT_CTL_MODE			0x0b
#define CAM_INQUIRE_INT_CTL_MODE		0x10b
#define MV1_INQUIRE_INTEGRATION_TIME		0x153
#define MV1_RESET_INTEGRATION_TIME		0x32153
#define MV1_INQUIRE_GRAB_ADR_OFFSETS            0x32154
#define FG_INQUIRE_REST_TIME			0x33232 
#define MV1_GET_PRIVATE_LUT			0x201
#define MV1_FREE_LUT				0x301

/* 4.10 Master Mode Transfer and Interrupt Handling */

#define MV1_SET_MASTER_MODE_TRANSFER		0x7a
#define MV1_SET_MASTER_MODE_TRANSFER_FIELD	0x7b
#define MV1_SET_MASTER_ENABLE  			0x7c
#define MV1_INQUIRE_MASTER_ENABLE		0x17c
#define MV1_SET_MASTER_LOOP			0x7d
#define MV1_INQUIRE_MASTER_LOOP			0x17d
#define MV1_SET_MASTER_WAIT_FRAME		0x1a
#define MV1_INQUIRE_MASTER_WAIT_FRAME		0x11a
#define MV1_CONNECT_INTERRUPT_TO_CALLBACK	0x7e
#define MV1_DISCONNECT_INTERRUPT_TO_CALLBACK	0x7f
#define MV1_INTERRUPT_PROCESSED			0x59
#define MV1_SET_FRAME_END_IRQ			0x5a
#define MV1_INQUIRE_FRAME_END_IRQ 		0x15a
#define MV1_SET_TRANSFER_END_IRQ		0x5b
#define MV1_INQUIRE_TRANSFER_END_IRQ		0x15b

#define MV1_WAIT_FRAME_END			0x34870
#define MV1_WAIT_TRANSFER_END			0x34871
#define MV1_INQUIRE_MACHINE_STATE		0x34878
#define MV1_INQUIRE_FRAME_END_COUNT		0x34879
#define MV1_INQUIRE_BAD_LIST_COUNT		0x3487a
#define MV1_INQUIRE_TRANSFER_END_COUNT		0x3487b

#define MV1_SNAP				0x83288
#define TEST1 0xba
#define TEST2 0xbb
#define TEST2_GET 0x1bb 
#endif // MV1DRV

